package com.btcpiyush.ads.google_applovin_unity_ads.open_ad

import androidx.annotation.NonNull
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleEventObserver
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.ProcessLifecycleOwner
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.EventChannel.StreamHandler
import io.flutter.plugin.common.MethodChannel.MethodCallHandler

/** Listens to changes in app foreground/background and forwards events to Flutter.  */
internal class AppStateNotifier(binaryMessenger: BinaryMessenger) : LifecycleEventObserver,
    MethodCallHandler, StreamHandler {
    private val methodChannel: MethodChannel
    private val eventChannel: EventChannel
    private var events: EventSink? = null

    /** Starts listening for app lifecycle changes.  */
    fun start() {
        ProcessLifecycleOwner.get().lifecycle.addObserver(this)
    }

    /** Stops listening for app lifecycle changes.  */
    fun stop() {
        ProcessLifecycleOwner.get().lifecycle.removeObserver(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        when (call.method) {
            "start" -> start()
            "stop" -> stop()
            else -> result.notImplemented()
        }
    }

    override fun onStateChanged(source: LifecycleOwner, event: Lifecycle.Event) {
        if (event == Lifecycle.Event.ON_START && events != null) {
            events?.success("foreground")
        } else if (event == Lifecycle.Event.ON_STOP && events != null) {
            events?.success("background")
        }
    }

    override fun onListen(arguments: Any?, events: EventSink?) {
        this.events = events
    }

    override fun onCancel(arguments: Any?) {
        events = null
    }

    companion object {
        private const val METHOD_CHANNEL_NAME =
            "dev.bitcoding.ads/app_state_method"
        private const val EVENT_CHANNEL_NAME =
            "dev.bitcoding.ads/app_state_event"
    }

    init {
        methodChannel = MethodChannel(binaryMessenger, METHOD_CHANNEL_NAME)
        methodChannel.setMethodCallHandler(this)
        eventChannel = EventChannel(binaryMessenger, EVENT_CHANNEL_NAME)
        eventChannel.setStreamHandler(this)
    }
}