package com.btcpiyush.ads.google_applovin_unity_ads.reward_ad

import android.app.Activity
import android.util.Log
import androidx.annotation.NonNull
import com.applovin.mediation.*
import com.applovin.mediation.ads.MaxRewardedAd
import com.btcpiyush.ads.google_applovin_unity_ads.GoogleApplovinUnityAdsPlugin
import com.btcpiyush.ads.google_applovin_unity_ads.ProgressDialog
import com.btcpiyush.ads.google_applovin_unity_ads.R
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppLovinSetting
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppSettings
import com.btcpiyush.ads.google_applovin_unity_ads.api.FacebookAdsSetting
import com.btcpiyush.ads.google_applovin_unity_ads.api.GoogleAdsSetting
import com.facebook.ads.Ad
import com.facebook.ads.RewardedInterstitialAdListener
import com.facebook.ads.RewardedInterstitialAd as FBRewardedAd
import com.google.android.gms.ads.AdError
import com.google.android.gms.ads.AdRequest
import com.google.android.gms.ads.FullScreenContentCallback
import com.google.android.gms.ads.LoadAdError
import com.google.android.gms.ads.OnUserEarnedRewardListener
import com.google.android.gms.ads.rewarded.RewardItem
import com.google.android.gms.ads.rewarded.RewardedAd
import com.google.android.gms.ads.rewarded.RewardedAdLoadCallback
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.util.*

class AdmobReward(
    private val flutterPluginBinding: FlutterPlugin.FlutterPluginBinding,
    private val act: Activity,
    private val settingResponse: AppSettings
) :
    MethodChannel.MethodCallHandler {

    companion object {
        val allAds: MutableMap<Int, RewardModel> = mutableMapOf()
        val TAG: String = "ADMOBREWARD ADS"
    }

    private fun getGoogleAdRequest(): AdRequest {
        return AdRequest.Builder().build()
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "call.ads.rewarded.isLoaded" -> {
                val id = call.argument<Int>("id") ?: 0
                val model = allAds[id] ?: return result.success(false)

                if (model.isLoaded)
                    result.success(true)
                else
                    result.success(false)
            }

            "call.ads.rewarded" -> {
                val id: Int = call.argument<Int?>("id") ?: 0

                showLog("id  ======  $id")

                if (allAds[id] == null) {
                    allAds[id] = RewardModel(
                        progressBar = ProgressDialog(
                            act,
                            settingResponse.adSetting?.adsLoadingText ?: "Ads will loading..."
                        )
                    )
                }

                val model = allAds[id]

                model?.let { model ->
                    model.pos = 0
                    loadRewardAds(id, model)
                }
            }
            "call.ads.rewarded.dispose" -> {
                val id = call.argument<Int>("id") ?: 0
                allAds.remove(id)
            }
            else -> result.notImplemented()
        }
    }

    private fun loadRewardAds(id: Int, model: RewardModel) {
        val pos = model.pos++
        showLog("pos ======== $pos   ${settingResponse.adsSequence.size}")
        if (settingResponse.adsSequence.size > 0 && settingResponse.adsSequence.size > pos) {
            when (settingResponse.adsSequence[pos].lowercase()) {
                act.getString(R.string.google_ads).lowercase() -> {
                    settingResponse.googleAds?.let { google_ads ->
                        if (google_ads.isRewardedAds == true) {
                            loadGoogleRewardAds(
                                id,
                                model,
                                google_ads
                            )
                            return@loadRewardAds
                        }
                    }
                }

                act.getString(R.string.facebook_ads).lowercase() -> {
                    settingResponse.facebookAds?.let { facebookAds ->
                        if (facebookAds.isRewardedAds == true) {
                            loadFacebookRewardAds(
                                id,
                                model,
                                facebookAds
                            )
                            return@loadRewardAds
                        }
                    }
                }

                act.getString(R.string.app_lovin).lowercase() -> {
                    settingResponse.appLovin?.let { appLovinAds ->
                        if (appLovinAds.isRewardedAds == true) {
                            loadAppLovinRewardAds(
                                id,
                                model,
                                appLovinAds
                            )
                            return@loadRewardAds
                        }
                    }

                }
            }
            loadRewardAds(id, model)
        } else {
            val adChannel = MethodChannel(
                flutterPluginBinding.binaryMessenger,
                "${GoogleApplovinUnityAdsPlugin.CHANNEL_REWARD_NAME}_$id"
            )
            model.isInProgress = false
            model.isLoaded = false
            model.progressBar.dismiss()
            adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_ERROR, null)
        }
    }

    private fun loadGoogleRewardAds(
        id: Int,
        model: RewardModel,
        it: GoogleAdsSetting
    ) {
        if (model.isInProgress == true) {
            return@loadGoogleRewardAds
        }

        if (model.rewardAd != null) {
            if(model.isLoaded)
                showRewardAds(id, model)
            return@loadGoogleRewardAds
        }

        val admobID = it.rewardedId ?: ""

        if (!admobID.isNullOrEmpty()) {
            model.isInProgress = true
            model.isLoaded = false
            model.progressBar.show()
            val adChannel = MethodChannel(
                flutterPluginBinding.binaryMessenger,
                "${GoogleApplovinUnityAdsPlugin.CHANNEL_REWARD_NAME}_$id"
            )
            showLog("$admobID is not empty ")
            val callback = object : RewardedAdLoadCallback() {
                override fun onAdLoaded(@NonNull rewardedAd: RewardedAd) {
                    super.onAdLoaded(rewardedAd)
                    model.isInProgress = false
                    model.isLoaded = true
                    model.progressBar.dismiss()
                    showRewardAds(id, model)
                    model.rewardAd?.fullScreenContentCallback =
                        object : FullScreenContentCallback() {
                            override fun onAdClicked() {
                                showLog("   --------  Reward onAdClicked ---   ")
                                super.onAdClicked()
                            }

                            override fun onAdDismissedFullScreenContent() {
                                super.onAdDismissedFullScreenContent()
                                model.rewardAd = null
                                model.progressBar.dismiss()
                                showLog("   -------- onAdDismissedFullScreenContent Reward Show  ")

                                if (model.isRewardVideoShowCompleted) {
                                    model.isRewardVideoShowCompleted = false
                                    adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS, null)
                                }
                            }

                            override fun onAdShowedFullScreenContent() {
                                super.onAdShowedFullScreenContent()

                                model.progressBar.dismiss()
                                showLog(" -------- onAdShowedFullScreenContent Reward Show  --- ")
                            }

                            override fun onAdFailedToShowFullScreenContent(adError: AdError) {
                                super.onAdFailedToShowFullScreenContent(adError)

                                model.progressBar.dismiss()
                                showLog(" -------- onAdFailedToShowFullScreenContent Reward Show  --- $adError")
                                loadRewardAds(id, model)
                            }

                            override fun onAdImpression() {
                                super.onAdImpression()
                                showLog(" -------- onAdImpression Reward Show  --- ")
                            }
                        }
                }

                override fun onAdFailedToLoad(loadAdError: LoadAdError) {
                    super.onAdFailedToLoad(loadAdError)
                    model.isInProgress = false
                    model.isLoaded = true
                    model.rewardAd = null
                    showLog(
                        " -------- onAdFailedToLoad Rewarded  --- $loadAdError"
                    )
                    loadRewardAds(id, model)
                }
            }
            RewardedAd.load(
                act, admobID, getGoogleAdRequest(),
                callback
            )
            return@loadGoogleRewardAds
        }

        loadRewardAds(id, model)
    }

    private fun loadFacebookRewardAds(
        id: Int,
        model: RewardModel,
        it: FacebookAdsSetting
    ) {
        if (model.isInProgress == true) {
            showLog("Reward Ad is Already Loading Progress...")
            return
        }

        if (model.fbRewardAd != null) {
            showLog("Reward Ad is Already Loading Completed")
            if (model.isLoaded) {
                showFacebookRewardAds(id, model)
            }
            return
        }

        val admobID = it.rewardedId ?: ""

        if (!admobID.isNullOrEmpty()) {
            model.isInProgress = true
            model.isLoaded = false
            model.progressBar.show()
            val adChannel = MethodChannel(
                flutterPluginBinding.binaryMessenger,
                "${GoogleApplovinUnityAdsPlugin.CHANNEL_REWARD_NAME}_$id"
            )
            showLog("$admobID is not empty ")

            val callback = object: RewardedInterstitialAdListener {
                override fun onError(p0: Ad?, p1: com.facebook.ads.AdError?) {
                    model.fbRewardAd = null
                    model.isInProgress = false
                    loadRewardAds(id, model)
                    showLog(
                        " -------- onAdFailedToLoad Rewarded  --- ${p1?.errorMessage}"
                    )
                }

                override fun onAdLoaded(p0: Ad?) {
                    model.isInProgress = false
                    model.isLoaded = true
                    model.progressBar.dismiss()
                    showFacebookRewardAds(id, model)
                }

                override fun onAdClicked(p0: Ad?) {

                }

                override fun onLoggingImpression(p0: Ad?) {

                }

                override fun onRewardedInterstitialCompleted() {
                    model.isRewardVideoShowCompleted = true
                }

                override fun onRewardedInterstitialClosed() {
                    model.progressBar.dismiss()
                    model.isInProgress = false
                    model.fbRewardAd = null
                    if (model.isRewardVideoShowCompleted) {
                        model.isRewardVideoShowCompleted = false
                        adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS, null)
                    }
                }

            }

            model.fbRewardAd = FBRewardedAd(act, admobID)
            model.fbRewardAd?.let {
                it.loadAd(
                    it.buildLoadAdConfig().withAdListener(callback).build()
                )
                return@loadFacebookRewardAds
            }

        }

        loadRewardAds(id, model)

    }

    private fun loadAppLovinRewardAds(
        id: Int,
        model: RewardModel,
        it: AppLovinSetting
    ) {
        if (model.isInProgress == true) {
            showLog("Interestial Ad is Already Loading Progress...")
            return
        }

        if (model.appLovinRewardAd != null) {
            showLog("Interestial Ad is Already Loading Completed")
            if (model.isLoaded) {
                showAppLovinRewardAds(id, model)
            }
            return
        }

        val admobID = it.appLovinRewardedID ?: ""

        if (!admobID.isNullOrEmpty()) {
            model.isInProgress = true
            model.isLoaded = false
            model.progressBar.show()
            val adChannel = MethodChannel(
                flutterPluginBinding.binaryMessenger,
                "${GoogleApplovinUnityAdsPlugin.CHANNEL_REWARD_NAME}_$id"
            )
            showLog("$admobID is not empty ")
            val callback = object : MaxRewardedAdListener {

                override fun onAdLoaded(maxAd: MaxAd?) {
                    // Reset retry attempt
//                                  retryAttempt = 0
                    model.isInProgress = false
                    model.isLoaded = true
                    model.progressBar.dismiss()
                    showAppLovinRewardAds(id, model)
                }

                override fun onAdLoadFailed(
                    adUnitId: String?,
                    error: MaxError?
                ) {
                    showLog("onAdLoadFailed: AppLovin Ad Failed  ------ ")
                    model.appLovinRewardAd = null
                    model.isInProgress = false
                    model.isLoaded = false
                    model.progressBar.dismiss()
                    loadRewardAds(id, model)
                }

                override fun onAdDisplayFailed(
                    maxAd: MaxAd?,
                    error: MaxError?
                ) {
                    // Interstitial ad failed to display. AppLovin recommends that you load the next ad.
                    // mInterstitialAppLovinAd?.loadAd()
                    model.appLovinRewardAd = null
                    model.isInProgress = false
                    model.isLoaded = false
                    showLog("onAdDisplayFailed: AppLovin Ad Failed   ------    ")
                    model.progressBar.dismiss()
                    loadRewardAds(id, model)
                }

                override fun onRewardedVideoStarted(ad: MaxAd?) {

                }

                override fun onRewardedVideoCompleted(ad: MaxAd?) {
                    model.isRewardVideoShowCompleted = true
                }

                override fun onUserRewarded(ad: MaxAd?, reward: MaxReward?) {

                }

                override fun onAdDisplayed(maxAd: MaxAd?) {
                    model.progressBar.dismiss()
                }

                override fun onAdClicked(maxAd: MaxAd?) {

                }

                override fun onAdHidden(maxAd: MaxAd?) {
                    model.isInProgress = false
                    model.appLovinRewardAd = null
                    model.progressBar.dismiss()
                    if (model.isRewardVideoShowCompleted) {
                        model.isRewardVideoShowCompleted = false
                        adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS, null)
                    }
                }

            }

            GoogleApplovinUnityAdsPlugin.appLovinSDK?.let { appLovinSDK ->
                model.appLovinRewardAd = MaxRewardedAd.getInstance(admobID, appLovinSDK, act)

                model.appLovinRewardAd?.let {
                    it.setListener(callback)
                    it.loadAd()
                    return@loadAppLovinRewardAds
                }

            }
        }

        loadRewardAds(id, model)
    }

    private fun showRewardAds(
        id: Int,
        model: RewardModel
    ) {
        model.rewardAd?.let {
            try {
                it.show(act, OnUserEarnedRewardListener { rewardItem: RewardItem? ->
                    showLog("pragnesh palsana")
                    model.rewardAd = null
                    model.isRewardVideoShowCompleted = true
                })
                return@showRewardAds
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
        loadRewardAds(id, model)
    }

    private fun showFacebookRewardAds(
        id: Int,
        model: RewardModel
    ) {
        model.fbRewardAd?.let {
            showLog("Facebook Ads Show Completed")
            try {
                if (it.isAdLoaded && !it.isAdInvalidated) {
                    it.show()
                    return@showFacebookRewardAds
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }

        loadRewardAds(id, model)
    }

    private fun showAppLovinRewardAds(
        id: Int,
        model: RewardModel
    ) {
        model.appLovinRewardAd?.let {
            showLog("Applovin Ads Show Completed")
            if (it.isReady) {
                it.showAd()
                return@showAppLovinRewardAds
            }
        }

        loadRewardAds(id, model)

    }


    private fun showLog(error: String) {
        Log.e(TAG, error)
    }

}