import 'dart:async';

import 'package:flutter/services.dart';
import 'package:flutter/material.dart';
import 'package:google_applovin_unity_ads/events/admob_events.dart';
import 'package:google_applovin_unity_ads/interestial/interstitial_ad.dart';
import 'package:google_applovin_unity_ads/open/app_lifecycle_reactor.dart';
import 'package:google_applovin_unity_ads/open/open_ad.dart';
import 'package:google_applovin_unity_ads/reward/reward_ad.dart';
import 'banner/banner_ad_widget.dart';
import 'banner/controller.dart';
import 'native/controller.dart';
import 'native/native_ad_widget.dart';

class GoogleApplovinUnityAds {
  static const channelName = 'dev.bitcoding.ads/google_applovin_unity_ads';
  static const platform = MethodChannel(channelName);
  static InterstitialAd? _interestialAd;
  static RewardAd? _rewardAd;
  static OpenAd? _openAd;
  static AppLifecycleReactor? _appLifecycleReactor;

  static initialize(String json_data,
      {Function? callback = null, bool isOpenAdDefault = true}) async {
    GoogleApplovinUnityAds.platform.setMethodCallHandler(null);
    GoogleApplovinUnityAds.platform.setMethodCallHandler(
        (call) => handleCallback(call, callback: callback));

    await GoogleApplovinUnityAds.platform
        .invokeMethod('call.ads.api.setting', {"json_data": json_data});
  }

  static OpenAd? getOpenAd() {
    return _openAd;
  }

  static Future<void> handleCallback(MethodCall call,
      {Function? callback, bool isOpenAdDefault = true}) async {
    switch (call.method) {
      case 'SUCCESS':
        if (isOpenAdDefault) {
          _openAd ??= OpenAd(
              listener: (AdmobAdEvent event, Map<String, dynamic>? args) {
            GoogleApplovinUnityAds.handleEvent(event);
          });

          if (_openAd != null) {
            _openAd?.load();

            _appLifecycleReactor =
                AppLifecycleReactor(appOpenAdManager: _openAd!);

            _appLifecycleReactor?.listenToAppStateChanges();
          }
        }

        if (callback != null) {
          callback();
        }

        break;
      case 'ERROR':
        if (callback != null) {
          callback();
        }
        break;
    }
  }

  static showIntraAds({Function? callback = null}) async {
    _interestialAd = InterstitialAd(
        listener: (AdmobAdEvent event, Map<String, dynamic>? args) {
      GoogleApplovinUnityAds.handleEvent(event, callback: callback);
    });

    _interestialAd?.load();
  }

  static Future<void> showRewardAds(
      {Function? callback = null, bool showIntraAds = true}) async {
    _rewardAd =
        RewardAd(listener: (AdmobAdEvent event, Map<String, dynamic>? args) {
      print("_-----_----_----_------_----_  $event");
      _handleRewardEvent(event, showIntraAds, callback: callback);
    });

    _rewardAd?.load();
  }

  static handleEvent(AdmobAdEvent event, {Function? callback = null}) {
    switch (event) {
      case AdmobAdEvent.success:
      case AdmobAdEvent.error:
        if (callback != null) {
          callback();
        }
        print('New Event Callback $event !');
        break;
      default:
        break;
    }
  }

  static _handleRewardEvent(AdmobAdEvent event, bool isShowIntraAds,
      {Function? callback = null}) {
    print('New Event Reward Callback $event !');
    switch (event) {
      case AdmobAdEvent.success:
        if (callback != null) {
          callback();
        }
        break;
      case AdmobAdEvent.error:
        if (isShowIntraAds) {
          Future.delayed(const Duration(milliseconds: 5), () {
            showIntraAds(callback: callback);
          });
        } else {
          if (callback != null) {
            callback();
          }
        }
        break;
      default:
        break;
    }
  }

  static bannerAds(
      {BannerSize bannerSize = BannerSize.BANNER,
      String adLoadingText = "Ad is loading.......",
      Color? boxDecorationcolor = const Color(0xFFFFFFFF),
      Color boxBordercolor = const Color(0xFFFFFFFF),
      TextStyle? style = const TextStyle(fontSize: 16, color: Colors.black),
      Widget? error = const SizedBox(
        height: 0,
      )}) {
    return Stack(
      children: [
        Container(
          decoration: BoxDecoration(
              color: boxDecorationcolor,
              border: Border.all(color: boxBordercolor)),
          height: bannerSize.size.height,
          child: Center(
            child: BannerAd(
              builder: (context, child) {
                return Container(
                  color: Colors.black,
                  child: child,
                );
              },
              loading: SizedBox(
                  height: bannerSize.size.height,
                  child: Center(
                      child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          crossAxisAlignment: CrossAxisAlignment.center,
                          children: [
                        Transform.scale(
                          scale: 0.5,
                          child: const CircularProgressIndicator(
                              color: Colors.black),
                        ),
                        SizedBox(
                          width: 7,
                        ),
                        Text(adLoadingText, style: style)
                      ]))),
              // ignore: deprecated_member_use
              size: bannerSize,
            ),
          ),
        )
      ],
    );
  }

  static nativeAds(NativeSize nativeSize, String nativeSizeType,
      {String adLoadingText = "Ad is loading.......",
      Color? boxDecorationcolor = const Color(0xFFFFFFFF),
      Color boxBordercolor = const Color(0xFFFFFFFF),
      String button_background_color = "#000000",
      String button_text_color = "#FFFFFF",
      TextStyle? style = const TextStyle(fontSize: 16, color: Colors.black),
      Widget? error = const SizedBox(
        height: 0,
      )}) {
    return Container(
      height: nativeSize.size.height,
      alignment: Alignment.center,
      margin: EdgeInsets.only(left: 2, right: 2),
      decoration: BoxDecoration(
          color: boxDecorationcolor, border: Border.all(color: boxBordercolor)),
      child: Padding(
        padding: EdgeInsets.all(1),
        child: Center(
          child: NativeAd(
            action_button_background_color: button_background_color,
            action_button_text_color: button_text_color,
            nativeSizeType: nativeSizeType,
            builder: (context, child) {
              return Container(
                alignment: Alignment.center,
                color: Colors.black,
                child: child,
              );
            },
            loading: SizedBox(
                height: nativeSize.size.height,
                child: Center(
                    child: Row(
                        mainAxisAlignment: MainAxisAlignment.center,
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                      Transform.scale(
                        scale: 0.5,
                        child: const CircularProgressIndicator(
                            color: Colors.black),
                      ),
                      SizedBox(
                        width: 8,
                      ),
                      Text(adLoadingText, style: style)
                    ]))),
            size: nativeSize,
            error: error,
          ),
        ),
      ),
    );
  }
}
